package jp.gr.java_conf.abagames.bulletml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * <b>Action</b> is generated from bulletml.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="action">
 *   <optional>
 *     <attribute name="label">
 *       <data type="string"/>
 *     </attribute>
 *   </optional>
 *   <zeroOrMore>
 *     <choice>
 *       <ref name="repeat"/>
 *       <ref name="fireElm"/>
 *       <ref name="changeSpeed"/>
 *       <ref name="changeDirection"/>
 *       <ref name="accel"/>
 *       <ref name="wait"/>
 *       <ref name="vanish"/>
 *       <ref name="actionElm"/>
 *     </choice>
 *   </zeroOrMore>
 * </element>
 * -->
 * <!-- for javadoc -->
 * <pre> &lt;element name="action"&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="label"&gt;
 *       &lt;data type="string"/&gt;
 *     &lt;/attribute&gt;
 *   &lt;/optional&gt;
 *   &lt;zeroOrMore&gt;
 *     &lt;choice&gt;
 *       &lt;ref name="repeat"/&gt;
 *       &lt;ref name="fireElm"/&gt;
 *       &lt;ref name="changeSpeed"/&gt;
 *       &lt;ref name="changeDirection"/&gt;
 *       &lt;ref name="accel"/&gt;
 *       &lt;ref name="wait"/&gt;
 *       &lt;ref name="vanish"/&gt;
 *       &lt;ref name="actionElm"/&gt;
 *     &lt;/choice&gt;
 *   &lt;/zeroOrMore&gt;
 * &lt;/element&gt;
 * </pre>
 *
 * @version bulletml.rng
 * @author  Relaxer 0.17 (http://www.relaxer.org)
 */
public class Action implements java.io.Serializable, IActionElmChoice, IActionChoice, IBulletmlChoice {
    private String label_;
    // List<IActionChoice>
    private java.util.List content_ = new java.util.ArrayList();

    /**
     * Creates a <code>Action</code>.
     *
     */
    public Action() {
    }

    /**
     * Creates a <code>Action</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Action(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Action</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Action(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Action</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Action(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Action</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Action(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Action</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Action(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Action</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Action(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Action</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Action(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Action</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Action(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Action</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Action(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Action</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Action</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Action</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        label_ = URelaxer.getAttributePropertyAsString(element, "label");
        content_.clear();
        while (true) {
            if (FireRef.isMatch(stack)) {
                addContent(new FireRef(stack));
            } else if (ChangeSpeed.isMatch(stack)) {
                addContent(new ChangeSpeed(stack));
            } else if (Repeat.isMatch(stack)) {
                addContent(new Repeat(stack));
            } else if (Fire.isMatch(stack)) {
                addContent(new Fire(stack));
            } else if (ChangeDirection.isMatch(stack)) {
                addContent(new ChangeDirection(stack));
            } else if (Accel.isMatch(stack)) {
                addContent(new Accel(stack));
            } else if (Action.isMatch(stack)) {
                addContent(new Action(stack));
            } else if (ActionRef.isMatch(stack)) {
                addContent(new ActionRef(stack));
            } else if (Wait.isMatch(stack)) {
                addContent(new Wait(stack));
            } else if (Vanish.isMatch(stack)) {
                addContent(new Vanish(stack));
            } else {
                break;
            }
        }
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("action");
        int size;
        if (this.label_ != null) {
            URelaxer.setAttributePropertyByString(element, "label", this.label_);
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IActionChoice value = (IActionChoice)this.content_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Action</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Action</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Action</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Action</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Action</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Action</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>label</b>.
     *
     * @return String
     */
    public final String getLabel() {
        return (label_);
    }

    /**
     * Sets the String property <b>label</b>.
     *
     * @param label
     */
    public final void setLabel(String label) {
        this.label_ = label;
    }

    /**
     * Gets the IActionChoice property <b>content</b>.
     *
     * @return IActionChoice[]
     */
    public final IActionChoice[] getContent() {
        IActionChoice[] array = new IActionChoice[content_.size()];
        return ((IActionChoice[])content_.toArray(array));
    }

    /**
     * Sets the IActionChoice property <b>content</b>.
     *
     * @param content
     */
    public final void setContent(IActionChoice[] content) {
        this.content_.clear();
        for (int i = 0;i < content.length;i++) {
            addContent(content[i]);
        }
    }

    /**
     * Sets the IActionChoice property <b>content</b>.
     *
     * @param content
     */
    public final void setContent(IActionChoice content) {
        this.content_.clear();
        addContent(content);
    }

    /**
     * Adds the IActionChoice property <b>content</b>.
     *
     * @param content
     */
    public final void addContent(IActionChoice content) {
        this.content_.add(content);
    }

    /**
     * Adds the IActionChoice property <b>content</b>.
     *
     * @param content
     */
    public final void addContent(IActionChoice[] content) {
        for (int i = 0;i < content.length;i++) {
            addContent(content[i]);
        }
    }

    /**
     * Gets number of the IActionChoice property <b>content</b>.
     *
     * @return int
     */
    public final int getContentCount() {
        return (content_.size());
    }

    /**
     * Gets the IActionChoice property <b>content</b> by index.
     *
     * @param index
     * @return IActionChoice
     */
    public final IActionChoice getContent(int index) {
        return ((IActionChoice)content_.get(index));
    }

    /**
     * Sets the IActionChoice property <b>content</b> by index.
     *
     * @param index
     * @param content
     */
    public final void setContent(int index, IActionChoice content) {
        this.content_.set(index, content);
    }

    /**
     * Adds the IActionChoice property <b>content</b> by index.
     *
     * @param index
     * @param content
     */
    public final void addContent(int index, IActionChoice content) {
        this.content_.add(index, content);
    }

    /**
     * Remove the IActionChoice property <b>content</b> by index.
     *
     * @param index
     */
    public final void removeContent(int index) {
        this.content_.remove(index);
    }

    /**
     * Remove the IActionChoice property <b>content</b> by object.
     *
     * @param content
     */
    public final void removeContent(IActionChoice content) {
        this.content_.remove(content);
    }

    /**
     * Clear the IActionChoice property <b>content</b>.
     *
     */
    public final void clearContent() {
        this.content_.clear();
    }

    /**
     * Makes a XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes a XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<action");
        if (label_ != null) {
            buffer.append(" label=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.append("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IActionChoice value = (IActionChoice)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.append(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IActionChoice value = (IActionChoice)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</action>");
    }

    /**
     * Makes a XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<action");
        if (label_ != null) {
            buffer.print(" label=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getLabel())));
            buffer.print("\"");
        }
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IActionChoice value = (IActionChoice)this.content_.get(i);
            value.makeTextAttribute(buffer);
        }
        buffer.print(">");
        size = this.content_.size();
        for (int i = 0;i < size;i++) {
            IActionChoice value = (IActionChoice)this.content_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</action>");
    }

    /**
     * Makes a XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes a XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getLabelAsString() {
        return (URelaxer.getString(getLabel()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setLabelByString(String string) {
        setLabel(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Action</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "action")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        while (true) {
            if (FireRef.isMatchHungry(target)) {
                $match$ = true;
            } else if (ChangeSpeed.isMatchHungry(target)) {
                $match$ = true;
            } else if (Repeat.isMatchHungry(target)) {
                $match$ = true;
            } else if (Fire.isMatchHungry(target)) {
                $match$ = true;
            } else if (ChangeDirection.isMatchHungry(target)) {
                $match$ = true;
            } else if (Accel.isMatchHungry(target)) {
                $match$ = true;
            } else if (Action.isMatchHungry(target)) {
                $match$ = true;
            } else if (ActionRef.isMatchHungry(target)) {
                $match$ = true;
            } else if (Wait.isMatchHungry(target)) {
                $match$ = true;
            } else if (Vanish.isMatchHungry(target)) {
                $match$ = true;
            } else {
                break;
            }
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Action</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Action</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
